{% load i18n %}
<footer class="misago-footer">
  <div class="container">
    <div class="footer-content">

      <noscript>
        <p class="noscript-message">
          <i class="material-icon hidden-xs hidden-sm">info_outline</i>
          {% trans "For complete experience please enable JavaScript." %}
        </p>
      </noscript>

      {% if TERMS_OF_SERVICE_URL or PRIVACY_POLICY_URL or misago_settings.forum_footnote %}
        <ul class="list-inline footer-nav">
        {% if misago_settings.forum_footnote %}
          <li class="site-footnote">
            {{ misago_settings.forum_footnote }}
          </li>
        {% endif %}
        {% if TERMS_OF_SERVICE_URL %}
          <li>
            <a href="{{ TERMS_OF_SERVICE_URL }}">{% trans "Terms of service" %}</a>
          </li>
        {% endif %}
        {% if PRIVACY_POLICY_URL %}
          <li>
            <a href="{{ PRIVACY_POLICY_URL }}">{% trans "Privacy policy" %}</a>
          </li>
        {% endif %}
        </ul>
      {% endif %}

      <a href="http://misago-project.org" class="misago-branding">
        powered by <strong>misago</strong>
      </a>

    </div>
  </div>
</footer>
