{% extends "misago/admin/generic/form.html" %}
{% load i18n misago_forms %}


{% block title %}
{% trans "Request new data downloads" %} | {{ active_link.name }} | {{ block.super }}
{% endblock title %}


{% block page-target %}
{% trans "Request new data downloads" %}
{% endblock page-target %}


{% block form-header %}
<h1>
  {% trans "Request new data downloads" %}
</h1>
{% endblock %}


{% block form-extra %}
class="form-horizontal"
{% endblock form-extra%}


{% block form-body %}
<div class="form-body">
  <fieldset>
    {% form_row form.user_identifiers %}
  </fieldset>
</div>
{% endblock form-body %}


{% block form-footer-class %}
col-md-offset-3
{% endblock form-footer-class %}
